==> kuromojure-0.6.2-20140721.195424-40.jar: kuro/fude.clj <==
(ns kuro.fude
  (:use benri.kuro)
  (:require [clojure.string :as str]
            [clojure.core.match :refer [match]]))
;(->> (with-tokenizer :learning
;  (tokenize "そして　つい に真のキラと目された火口を追い詰めるに至ったのです")) (map #(select-keys % [:surface :part-of-speech])))
(defonce training-data
  (str "後 に　就いて　言って　下さい。 もう　一度。 全部　一緒 に。 はじめ から。"
       "次 を　どうぞ。 よく　できる。 おはよう　ございます。 どうも　ありがとう　ございます。 "
       "どう　いたしまして。 お元気　です　か。 いかが　です　か。 元気　です。 まあまあ　です。 "
       "おかげさまで。 お蔭　様 で。 行って　来ます。 行って　いらっしゃい。 行ってらっしゃい。 "
       "ただいま。 お帰りなさい。 Ｌ　です。 まだまだ　暑い　日 が　続きます　ね。 "
       "でも　クーラー の　温度 の　下げすぎ は　体 に　よく　ありません。 "
       "今夜 は 「きんようとくべつ　ロード　ショー」と　題して。 皆さん　お待ち　かね。 "
       "「デス　ノート　リライト　２　Ｌ を　継ぐ者」を　お送り　いたします。 続編　であり。 "
       "日本 に　行きます。 必ず　２７　度前後 に　して。 "
       "冷やしすぎない　よう に　して　ください。 皆さん　お待ち　かね。 死んだ　あと。 "
       "昨年　放送　された　「リライト　幻視　する　神」の　続編　であり。 "
       "キラ と　私 の　後継　者 を　名乗る　人物 が　戦う　お話　だ　そう　です。 "
       "ここ から は　前作 を　ご覧 に　なっていない　方 の　ため に。"
       "それ　じゃあ　ワタリ　始めて　くれ。 いい　くらい の　真面目な　優等生　だ　よ。 "
       "デスノート を　拾った　こと を　きっかけ に、 "
       "あらゆる　人々 を　翻弄　する　孤高 の　殺人者。 "
       "キラ　事件　参考人 と　して　拘束　された が　"
       "その　最中　別 の　キラ が　現れ　動き　始めた　ため、 "
       "キラ　捜査 に　協力　する　こと と　なる。 新世界 の　神 と　なる。 "
       "Ｌ　キラ を　追う　世界　的　探偵。 キラ　事件　により　初めて　表舞台 に　立つ。 "
       "明晰な　頭脳 と　迅速な　行動力 で、 "
       "私 は　夜神君 を　キラ　じゃない　かと　疑っているんです。 "
       "かいつまん で　お話し　しましょう。 この　物語 の　主人公　夜　神月、 "
       "キラ と　して　暗躍、 時 に　協力し　時 に　傍観し、 持つ第　２ の　キラ。 "
       "女優 と　いった。 夜　神月 に　協力　する。 にっぽん　そうさほんぶ、 刑事たち、 "
       "混成　チーム で、 ヨツバ　という　企業 に、 そして　つい に、 "
       "そして　つい に　真 の　キラ と　目　された"))

(defn some-space
  "Returns space surfaceForm from token, or nil if token isn't any space."
  [token] (some #{(:surfaceForm token)} [" " "　"]))

(defn inject-space-info
  "Inserts space information (:prev-space/:next-space) into a token if the
   other token is a space."
  [prev-token next-token]
  [(or (some->> (some-space next-token) (assoc prev-token :next-space))
  	   prev-token)
   (or (some->> (some-space prev-token) (assoc next-token :prev-space))
   	   next-token)])

(defn- filter-by-some-part-of-speech
  [n {:keys [partOfSpeech]} tokens]
  (let [take-n (partial take n)
        split-speech #(str/split % #",")
        partOfSpeech (split-speech partOfSpeech)
        equal-speech #(= (take-n partOfSpeech) (-> % :partOfSpeech take-n))
        f (comp equal-speech #(update % :partOfSpeech split-speech))]
  	(seq (filter f tokens))))

(defn- filter-by-surface
  [{:keys [surfaceForm]} tokens]
  (seq (filter #(= (:surfaceForm %) surfaceForm) tokens)))

(defmulti elect-by (fn [& args] (first args)))

(defmethod elect-by :surface-and-part-of-speech
  [_ n token tokens]
  (let [take-n #(take n %)]
  	(->> (filter-by-some-part-of-speech n token tokens)
  		   (filter-by-surface token))))

(defmethod elect-by :part-of-speech [_ n token tokens]
  (filter-by-some-part-of-speech n token tokens))

(defmethod elect-by :surface [_ token tokens]
  (filter-by-surface token tokens))

(defn find-token-candidates
  [token the-model]
  "Accepts token and model returning a list of token candidates."
  (or (elect-by :surface-and-part-of-speech 4 token the-model)
      (elect-by :surface-and-part-of-speech 3 token the-model)
      (elect-by :surface-and-part-of-speech 2 token the-model)
      (elect-by :part-of-speech 4 token the-model)
      (elect-by :surface-and-part-of-speech 1 token the-model)
      (elect-by :part-of-speech 3 token the-model)
      (elect-by :part-of-speech 2 token the-model)
      (elect-by :surface token the-model)
      (elect-by :part-of-speech 1 token the-model)))

(defn remove-token-dups
  [tokens]
  (let [merge-dups (fn [prev nekt]
                     (if (= (:position prev) (:position nekt))
                       (merge prev nekt) [prev nekt]))]
    (loop [t (vec tokens) r []]
      (match [t r]
        [[] _] (reverse r)
        [[h1 & r1] []] (recur r1 [h1])
        [[h1 & r1] [h2 & r2]]
          (recur r1 (-> (merge-dups h1 h2) (cons r2) flatten vec))))))

==> kuromojure-0.6.2-20140721.232928-41.jar: kuro/fude.clj <==
(ns kuro.fude
  (:use benri.kuro)
  (:require [clojure.string :as str]
            [clojure.core.match :refer [match]]))
;(->> (with-tokenizer :learning
;  (tokenize "そして　つい に真のキラと目された火口を追い詰めるに至ったのです")) (map #(select-keys % [:surface :part-of-speech])))
(defonce training-data
  (str "後 に　就いて　言って　下さい。 もう　一度。 全部　一緒 に。 はじめ から。"
       "次 を　どうぞ。 よく　できる。 おはよう　ございます。 どうも　ありがとう　ございます。 "
       "どう　いたしまして。 お元気　です　か。 いかが　です　か。 元気　です。 まあまあ　です。 "
       "おかげさまで。 お蔭　様 で。 行って　来ます。 行って　いらっしゃい。 行ってらっしゃい。 "
       "ただいま。 お帰りなさい。 Ｌ　です。 まだまだ　暑い　日 が　続きます　ね。 "
       "でも　クーラー の　温度 の　下げすぎ は　体 に　よく　ありません。 "
       "今夜 は 「きんようとくべつ　ロード　ショー」と　題して。 皆さん　お待ち　かね。 "
       "「デス　ノート　リライト　２　Ｌ を　継ぐ者」を　お送り　いたします。 続編　であり。 "
       "日本 に　行きます。 必ず　２７　度前後 に　して。 "
       "冷やしすぎない　よう に　して　ください。 皆さん　お待ち　かね。 死んだ　あと。 "
       "昨年　放送　された　「リライト　幻視　する　神」の　続編　であり。 "
       "キラ と　私 の　後継　者 を　名乗る　人物 が　戦う　お話　だ　そう　です。 "
       "ここ から は　前作 を　ご覧 に　なっていない　方 の　ため に。"
       "それ　じゃあ　ワタリ　始めて　くれ。 いい　くらい の　真面目な　優等生　だ　よ。 "
       "デスノート を　拾った　こと を　きっかけ に、 "
       "あらゆる　人々 を　翻弄　する　孤高 の　殺人者。 "
       "キラ　事件　参考人 と　して　拘束　された が　"
       "その　最中　別 の　キラ が　現れ　動き　始めた　ため、 "
       "キラ　捜査 に　協力　する　こと と　なる。 新世界 の　神 と　なる。 "
       "Ｌ　キラ を　追う　世界　的　探偵。 キラ　事件　により　初めて　表舞台 に　立つ。 "
       "明晰な　頭脳 と　迅速な　行動力 で、 "
       "私 は　夜神君 を　キラ　じゃない　かと　疑っているんです。 "
       "かいつまん で　お話し　しましょう。 この　物語 の　主人公　夜　神月、 "
       "キラ と　して　暗躍、 時 に　協力し　時 に　傍観し、 持つ第　２ の　キラ。 "
       "女優 と　いった。 夜　神月 に　協力　する。 にっぽん　そうさほんぶ、 刑事たち、 "
       "混成　チーム で、 ヨツバ　という　企業 に、 そして　つい に、 "
       "そして　つい に　真 の　キラ と　目　された。 目 を　持つ第　２ の　キラ。 "
       "彼 の　ため　なら ば　何でも　する　という　魔性 の　持ち主。 "))

(defn some-space
  "Returns space surfaceForm from token, or nil if token isn't any space."
  [token] (some #{(:surfaceForm token)} [" " "　"]))

(defn inject-space-info
  "Inserts space information (:prev-space/:next-space) into a token if the
   other token is a space."
  [prev-token next-token]
  [(or (some->> (some-space next-token) (assoc prev-token :next-space))
  	   prev-token)
   (or (some->> (some-space prev-token) (assoc next-token :prev-space))
   	   next-token)])

(defn- filter-by-some-part-of-speech
  [n {:keys [partOfSpeech]} tokens]
  (let [take-n (partial take n)
        split-speech #(str/split % #",")
        partOfSpeech (split-speech partOfSpeech)
        equal-speech #(= (take-n partOfSpeech) (-> % :partOfSpeech take-n))
        f (comp equal-speech #(update % :partOfSpeech split-speech))]
  	(seq (filter f tokens))))

(defn- filter-by-surface
  [{:keys [surfaceForm]} tokens]
  (seq (filter #(= (:surfaceForm %) surfaceForm) tokens)))

(defmulti elect-by (fn [& args] (first args)))

(defmethod elect-by :surface-and-part-of-speech
  [_ n token tokens]
  (let [take-n #(take n %)]
  	(->> (filter-by-some-part-of-speech n token tokens)
  		   (filter-by-surface token))))

(defmethod elect-by :part-of-speech [_ n token tokens]
  (filter-by-some-part-of-speech n token tokens))

(defmethod elect-by :surface [_ token tokens]
  (filter-by-surface token tokens))

(defn find-token-candidates
  [token the-model]
  "Accepts token and model returning a list of token candidates."
  (or (elect-by :surface-and-part-of-speech 4 token the-model)
      (elect-by :surface-and-part-of-speech 3 token the-model)
      (elect-by :surface-and-part-of-speech 2 token the-model)
      (elect-by :part-of-speech 4 token the-model)
      (elect-by :surface-and-part-of-speech 1 token the-model)
      (elect-by :part-of-speech 3 token the-model)
      (elect-by :part-of-speech 2 token the-model)
      (elect-by :surface token the-model)
      (elect-by :part-of-speech 1 token the-model)))

(defn remove-token-dups
  [tokens]
  (let [merge-dups (fn [prev nekt]
                     (if (= (:position prev) (:position nekt))
                       (merge prev nekt) [prev nekt]))]
    (loop [t (vec tokens) r []]
      (match [t r]
        [[] _] (reverse r)
        [[h1 & r1] []] (recur r1 [h1])
        [[h1 & r1] [h2 & r2]]
          (recur r1 (-> (merge-dups h1 h2) (cons r2) flatten vec))))))
